/*
 * RSS v2.2 1999/06/01 
 *
 * Author: Liu Rui
 * 
 * RSS (Regular Statement String) is a development tool for software
 * application venders or developers to build their friendly user interfaces or
 * represent their informations.
 *  
 * You may use this software freely and may ask for help from Liu Rui as you 
 * meet any technical problems about it. But Liu Rui does not guarantee your 
 * applications to be well done.
 */

/******************************************************************************/
#ifndef _INC_RSS_H
#define _INC_RSS_H

#ifdef  __cplusplus
extern "C" {
#endif


/******************************************************************************/
/*---------------------------- HEADER FILES ----------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(WIN32)

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define RSSI_DLL_EXPORT _declspec(dllexport)

#else

#define RSSI_DLL_EXPORT

#endif


/******************************************************************************/
/*--------------------------------- DEFINES ----------------------------------*/

/* Special Characters */
#define RSS_END_MARK		'\0'
#define RSS_STM_MARK		'\n'
#define RSS_KEY_MARK		'~'
#define RSS_MUL_MARK		'|'
#define RSS_ESC_MARK		'`'

#define RSS_END_ESCC		'0'
#define RSS_STM_ESCC		'1'
#define RSS_KEY_ESCC		'2'
#define RSS_MUL_ESCC		'3'
#define RSS_ESC_ESCC		'4'

#define RSS_END_MARK_STR	"\x00"
#define RSS_STM_MARK_STR	"\n"
#define RSS_KEY_MARK_STR	"~"
#define RSS_MUL_MARK_STR	"|"
#define RSS_ESC_MARK_STR	"`"

/* Limitations */
#define RSS_MAX_NAMR_LEN	63
#define RSS_MAX_KEY_LEN		100
#define RSS_EXTRA_SIZE		128
#define RSS_MIN_UNIT_SIZE	2048
#define RSS_DEF_MAX_VALUE_LEN	4096
#define RSS_DEF_MAX_LEN		33554432
#define RSS_MAX_ARRAY_ITEM_NUM	1000


/* Return Values */
#define RSS_RT_SUCCESS			0
#define RSS_RT_INVALID			-1
#define RSS_RT_RSS_TOO_LONG		-100
#define RSS_RT_RSS_NULL			-101
#define RSS_RT_RSS_INVALID		-102
#define RSS_RT_RSS_ARRAYITEMNUM_INVALID	-103
#define RSS_RT_KEY_TOO_LONG		-200
#define RSS_RT_KEY_EMPTY		-201
#define RSS_RT_KEY_BAD_CHAR		-202
#define RSS_RT_VALUE_NOSPACE		-300
#define RSS_RT_VALUE_TOO_LONG		-301
#define RSS_RT_VALUE_NULL		-302
#define RSS_RT_VALUE_INVALID		-303
#define RSS_RT_RSSI_ERROR		-400
#define RSS_RT_FILE_ERROR		-500
#define RSS_RT_JVM_ERROR		-600
#define RSS_RT_LOCATE_CLS_ERROR 	-601
#define RSS_RT_JAVA_EXCEPTION		-602
#define RSS_RT_UNKNOWN_ACTION		-700
#define RSS_RT_BAD_HANDLE_STATUS	-701


/* Options */
#define RSS_OP_KEY_FAMILY_GET	1
#define RSS_OP_KEY_MULTI_LEVEL	2


/* Status Values */
#define RSS_STATUS_NONE		0
#define RSS_STATUS_START	1


/* RSST */
struct _tagRsst {
	char *Rss, *Buf, *Sto;
	char Table[RSS_MAX_NAMR_LEN+1], Tag[8];
	int Unit, Size, MaxLen, MaxValueLen, MaxArrayItemNum, Vlen;
	unsigned int Option;
	int Cn, Rn, Sr;
	int Rt, Status;
	void *Handle, *hLib, *Link;
	void (*Rssi)(struct _tagRsst *, char *);
};

typedef struct _tagRsst Rsst;

/* ChrPtr */
typedef char * ChrPtr;


/******************************************************************************/
/*-------------------------------- FUNCTIONS ---------------------------------*/

/*--- ChrPtr ---*/
void    ChrPtrFree(ChrPtr value);
int      ChrPtrLen(ChrPtr value);


/*--- Rss Basic ---*/
int RssValid(char *rss);
int RssValidKey(char *key);


/*--- Rsst Basic ---*/
void RsstInit (Rsst *rsst, int unit_size);
void RsstClose(Rsst *rsst);
int  RsstCheck(Rsst *rsst);

char * RsstGetRss0(Rsst *rsst);
ChrPtr RsstGetRss (Rsst *rsst);
void RsstSetRss(Rsst *rsst, char *rss);
void RsstSetRssNum(Rsst *rsst, char *rss1, int n);
void RsstClear(Rsst *rsst);

void RsstRssLoad(Rsst *rsst, char *filename);
void RsstRssDump(Rsst *rsst, char *filename);

int  RsstGetLen(Rsst *rsst);
int  RsstGetMaxLen(Rsst *rsst);
void RsstSetMaxLen(Rsst *rsst, int max_len);
int  RsstGetMaxValueLen(Rsst *rsst);
void RsstSetMaxValueLen(Rsst *rsst, int max_len);
int  RsstGetUnitSize(Rsst *rsst);
void RsstSetUnitSize(Rsst *rsst, int unit_size);
unsigned int RsstGetOption(Rsst *rsst, unsigned int opt);
void RsstSetOption(Rsst *rsst, unsigned int opt, unsigned int f);
int  RsstGetMaxArrayItemNum(Rsst *rsst);
void RsstSetMaxArrayItemNum(Rsst *rsst, int max_num);

int  RsstGetRtCode(Rsst *rsst);
void RsstSetRtCode(Rsst *rsst, int RtCode);
int  RsstGetStatus(Rsst *rsst);
void RsstSetStatus(Rsst *rsst, int status);

Rsst* RsstGetHandle(Rsst *rsst);
void  RsstSetHandle(Rsst *rsst, Rsst *handle);
Rsst* RsstNewHandle  (Rsst *rsst);
void  RsstCloseHandle(Rsst *rsst);


/*--- Key/Value ---*/
ChrPtr RsstKeyGet (Rsst *rsst, char *key);
char * RsstKeyGet0(Rsst *rsst, char *key);
ChrPtr RsstKeyGetM (Rsst *rsst, char *key, int no);
char * RsstKeyGetM0(Rsst *rsst, char *key, int no);

void  RsstKeyPut (Rsst *rsst, char *key, void *value, int len);
void  RsstKeyPut0(Rsst *rsst, char *key, char *value);
void RsstKeyPutM (Rsst *rsst, char *key, void *value, int len, int no);
void RsstKeyPutM0(Rsst *rsst, char *key, char *value, int no);

int  RsstKeyGetInteger (Rsst *rsst, char *key);
int  RsstKeyGetMInteger(Rsst *rsst, char *key, int no);
void RsstKeyPutInteger (Rsst *rsst, char *key, int  value);
void RsstKeyPutMInteger(Rsst *rsst, char *key, int  value, int no);

void RsstKeyCopy      (Rsst *rsst1, Rsst *rsst2, char *key);
void RsstKeyCopyFamily(Rsst *rsst1, Rsst *rsst2, char *key0);

void RsstKeyList(Rsst *rsst);
void RsstKeyListFamily(Rsst *rsst, char *key0);

int  RsstKeyIfDef(Rsst *rsst, char *key); /* it's not 0 if defined */
void RsstKeyRemove(Rsst *rsst, char *key);

void RsstKeyCPointerLengthPut(Rsst *rsst, char *key, void  *ptr,  int  len);  /*convenient function only for C/C++*/
void RsstKeyCPointerLengthGet(Rsst *rsst, char *key, void **pptr, int *plen); /*convenient function only for C/C++*/


/*--- Array ---*/
void RsstArrayFetch(Rsst *rsst, char *key);
void RsstArrayNew  (Rsst *rsst);
void RsstArraySave (Rsst *rsst, char *key);

ChrPtr RsstArrayGet (Rsst *rsst, int no);
char * RsstArrayGet0(Rsst *rsst, int no);

void RsstArrayPut (Rsst *rsst, char *value, int len, int no);
void RsstArrayPut0(Rsst *rsst, char *value, int no);

int  RsstArrayGetInteger(Rsst *rsst, int no);
void RsstArrayPutInteger(Rsst *rsst, int value, int no);

int RsstArrayItemNum(Rsst *rsst);
void RsstArrayRemoveItem(Rsst *rsst, int no);


/*--- Table ---*/
void RsstTableCreate(Rsst *rsst, char *tbl_name);
void   RsstTableSave(Rsst *rsst, int col_num, int row_num, int start_row);
void   RsstTableOpen(Rsst *rsst, char *tbl_name);

int   RsstTableGetColNum(Rsst *rsst);
int   RsstTableGetRowNum(Rsst *rsst);
int RsstTableGetStartRow(Rsst *rsst);

void RsstTableFetchRow(Rsst *rsst, int row);
void   RsstTableNewRow(Rsst *rsst);
void  RsstTableSaveRow(Rsst *rsst, int row);

ChrPtr RsstTableGet (Rsst *rsst, int col);
char * RsstTableGet0(Rsst *rsst, int col);

void RsstTablePut (Rsst *rsst, char *value, int len, int col);
void RsstTablePut0(Rsst *rsst, char *value, int col);

int  RsstTableGetInteger(Rsst *rsst, int col);
void RsstTablePutInteger(Rsst *rsst, int  value, int col);


/*--- ETC ---*/
void StringFitIn(char *str, char *str0, int len);

int Locate(Rsst *rsst, char *dll_path, char *func_name);
void Unlocate(Rsst *rsst);


/******************************************************************************/


#ifdef  __cplusplus
}
#endif

#endif
/******************************************************************************/
